import subprocess
from Bio import SeqIO

def run_sarus(sarus_jar, fasta_input, pwm_file, sarus_output):
    with open(sarus_output, "w") as out_f:
        subprocess.run([
            "java", "-jar", sarus_jar, fasta_input, pwm_file, "besthit"
        ], stdout=out_f)
    print(f"✅ SARUS output saved to: {sarus_output}")

def parse_sarus_to_bed(sarus_output, bed_output, motif_length=20):
    bed_set = set()
    with open(sarus_output, "r") as infile:
        for line in infile:
            line = line.strip()
            if not line:
                continue
            if line.startswith(">"):
                header = line[1:]
                _, _, chrom, coords = header.split(":")
                region_start = int(coords.split("-")[0])
            else:
                score, rel_pos, strand = line.split()
                abs_start = region_start + int(rel_pos)
                abs_end = abs_start + motif_length
                if float(score) > 4:
                    bed_line = f"{chrom}\t{abs_start}\t{abs_end}\t{strand}\t{score}"
                    bed_set.add(bed_line)

    with open(bed_output, "w") as outfile:
        for line in sorted(bed_set):
            outfile.write(line + "\n")

    print(f"✅ BED file created: {bed_output} ({len(bed_set)} unique entries)")

if __name__ == "__main__":
    # === Paths ===
    window_fa = "/data/ddpanchenko/main_dom/Timofey_project/CTCFs_peak.fasta"
    sarus_jar = "/home/ddpanchenko/Tools/sarus/sarus-latest.jar"
    pwm_file = "/data/ddpanchenko/main_dom/hybrid_anopheles_human/motifs/CTCF/CTCF_HUMAN.H11MO.0.A.pwm"
    sarus_output = "/data/ddpanchenko/main_dom/Timofey_project/SARUS_ctcf_motifs.txt"
    bed_output = "/data/ddpanchenko/main_dom/Timofey_project/ctcf_motifs_with_pvalue4.bed"

    # === Run Pipeline ===
    print("---Running SARUS...")
    run_sarus(sarus_jar, window_fa, pwm_file, sarus_output)

    print("---Parsing SARUS output to BED...")
    parse_sarus_to_bed(sarus_output, bed_output)
